/**
 ******************************************************************************
 * @file    XNucleo53L1A2.h
 * @author  JS
 * @version V0.0.1
 * @date    15-January-2021
 * @brief   Header file for component XNucleo53L1A2
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2019 STMicroelectronics</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * *** NOTE : By default hardlinks U10, U11, U15 & U18, on the underside of
 *            the X-NUCELO-53L1A1 expansion board are not made/OFF.
 *            These links must be made to allow interrupts from the Satellite boards
 *            to be received.
 *            U11 and U18 must be made/ON to allow interrupts to be received from the
 *            INT_L & INT_R positions; or
 *            U10 and U15 must be made/ON to allow interrupts to be received from the
 *            Alternate INT_L & INT_R positions.
 *            The X_NUCLEO_53L1A2 library defaults to use the INT_L/INT_R positions.
 *            INT_L is available on expansion board Arduino Connector CN5, pin 1 as D8.
 *            Alternate INT_L is on CN5 Connector pin 2 as D9.
 *            INT_R is available on expansion board Arduino Connector CN9, pin 3 as D2.
 *            Alternate INT_R is on CN9 Connector pin 5 as D4.
 *            The pinouts are shown here : https://developer.mbed.org/components/X-NUCLEO-53L1A2/
 */

#ifndef __VL53L1_A2_NUCLEO_CLASS_H
#define __VL53L1_A2_NUCLEO_CLASS_H


/* Includes ------------------------------------------------------------------*/
#include "VL53L1CB.h"
#include "Stmpe1600.h"
//#include "DevI2C.h"
#include "ToF_I2C.h"


/** New device addresses */
#define NEW_SENSOR_CENTRE_ADDRESS   0x56

#define NEW_SENSOR_LEFT_ADDRESS         0x58
#define NEW_SENSOR_RIGHT_ADDRESS        0x5a


/* Classes--------------------------------------------------------------------*/

/* Classes -------------------------------------------------------------------*/
/** Class representing the X-NUCLEO-VL53L1A2 expansion board
 */
class XNucleo53L1A2
{
public:
    /** Constructor 1
    * @param[in] &i2c device I2C to be used for communication
    */
    XNucleo53L1A2(ToF_DevI2C *ext_i2c) : dev_i2c(ext_i2c)
    {
        
                printf("XNucleo53L1A2 %d z\n",ext_i2c);
        stmpe1600_exp0 = new Stmpe1600((DevI2C*)ext_i2c, (0x43 * 2));     // U21

        stmpe1600_exp1 = new Stmpe1600((DevI2C*)ext_i2c, (0x42 * 2));     // U19

        xshutdown_centre = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_15, (0x42 * 2));     // U19 on schematic
        sensor_centre = new VL53L1CB(dev_i2c, xshutdown_centre, A2);

        xshutdown_left = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_14, (0x43 * 2));     // U21 on schematic
        sensor_left = new VL53L1CB(dev_i2c, xshutdown_left, D8);

        xshutdown_right = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_15, (0x43 * 2));     // U21 on schematic
        sensor_right = new VL53L1CB(dev_i2c, xshutdown_right, D2);
    }  
    
    /** Constructor 2
     * @param[in] &i2c device I2C to be used for communication
     * @param[in] PinName gpio1_centre Mbed DigitalOut pin name to be used as a centre sensor GPIO_1 INT
     * @param[in] PinName gpio1_left Mbed DigitalOut pin name to be used as a left sensor GPIO_1 INT
     * @param[in] PinName gpio1_right Mbed DigitalOut pin name to be used as a right sensor GPIO_1 INT
     */
    XNucleo53L1A2(ToF_DevI2C *ext_i2c, PinName gpio1_centre,
                  PinName gpio1_left, PinName gpio1_right) : dev_i2c(ext_i2c)
    {
        stmpe1600_exp0 = new Stmpe1600((DevI2C*)ext_i2c, (0x43 * 2));     // U21

        stmpe1600_exp1 = new Stmpe1600((DevI2C*)ext_i2c, (0x42 * 2));     // U19
 //       printf("ToF_DevI2C constructor %d %d %d %d\n",ext_i2c,gpio1_centre,gpio1_left,gpio1_right);

        xshutdown_centre = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_15, (0x42 * 2));     // U19 on schematic
        sensor_centre = new VL53L1CB(dev_i2c, xshutdown_centre, gpio1_centre);

        xshutdown_left = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_14, (0x43 * 2));     // U21 on schematic
        sensor_left = new VL53L1CB(dev_i2c, xshutdown_left, gpio1_left);

        xshutdown_right = new Stmpe1600DigiOut((DevI2C*)dev_i2c, GPIO_15, (0x43 * 2));     // U21 on schematic
        sensor_right = new VL53L1CB(dev_i2c, xshutdown_right, gpio1_right);
    }    
    
    
    /** Destructor
     */
    ~XNucleo53L1A2()
    {
        printf("XNucleo53L1A2 destructor \n");
        if (xshutdown_centre != NULL) {
            delete xshutdown_centre;
            xshutdown_centre = NULL;
        }
        if (sensor_centre != NULL) {
            delete sensor_centre;
            sensor_centre = NULL;
        }
        if (xshutdown_left != NULL) {
            delete xshutdown_left;
            xshutdown_left = NULL;
        }
        if (sensor_left != NULL) {
            delete sensor_left;
            sensor_left = NULL;
        }
        if (xshutdown_right != NULL) {
            delete xshutdown_right;
            xshutdown_right = NULL;
        }
        if (sensor_right != NULL) {
            delete sensor_right;
            sensor_right = NULL;
        }

        delete stmpe1600_exp0;
        stmpe1600_exp0 = NULL;
        delete stmpe1600_exp1;
        stmpe1600_exp1 = NULL;
        _instance = NULL;
    }


    /**
     * @brief       Creates a singleton object instance
     * @param[in]   &i2c device I2C to be used for communication
     * @return      Pointer to the object instance
     */
    static XNucleo53L1A2 *instance(ToF_DevI2C *ext_i2c);

    /**
     * @brief       Creates a singleton object instance
     * @param[in]   &i2c device I2C to be used for communication
     * @param[in]   PinName gpio1_centre the pin connected to centre sensor INT
     * @param[in]   PinName gpio1_left the pin connected to left sensor INT
     * @param[in]   PinName gpio1_right the pin connected to right sensor INT
     * @return      Pointer to the object instance
     */
    static XNucleo53L1A2 *instance(ToF_DevI2C *ext_i2c, PinName gpio1_centre,
                                   PinName gpio1_left, PinName gpio1_right);

    /**
     * @brief       Initialize the board and sensors with deft values
     * @return      0 on success
     */
    int init_board();
    
    ToF_DevI2C *dev_i2c;
    
    VL53L1CB *sensor_centre;
    VL53L1CB *sensor_left;
    VL53L1CB *sensor_right;
    
    Stmpe1600 *stmpe1600_exp0;
    Stmpe1600 *stmpe1600_exp1;
    
    Stmpe1600DigiOut *xshutdown_centre;
    Stmpe1600DigiOut *xshutdown_left;
    Stmpe1600DigiOut *xshutdown_right;
    
private:
    static XNucleo53L1A2 *_instance;
};


#endif /* __VL53L1_A2_NUCLEO_CLASS_H */